<?php
// namespace components\com_responsivizer\libraries\framework\model;
/**
 * @package RESPONSIVIZER::FRAMEWORK::administrator::components::com_responsivizer
 * @subpackage framework
 * @subpackage model
 * @author Joomla! Extensions Store
 * @copyright (C) 2013 - Joomla! Extensions Store
 * @license GNU/GPLv2 http://www.gnu.org/licenses/gpl-2.0.html
 */
defined('_JEXEC') or die('Restricted access');
jimport('joomla.application.component.model');

/**
 * Base model responsibilities
 * 
 * @package RESPONSIVIZER::FRAMEWORK::administrator::components::com_responsivizer
 * @subpackage framework
 * @subpackage model
 * @since 2.0
 */
interface IResponsivizerModel {
	/**
	 * Main get data method
	 *
	 * @access public
	 * @return Object[]
	 */
	public function getData();

	/**
	 * Counter result set
	 *
	 * @access public
	 * @return int
	 */
	public function getTotal();

	/**
	 * Load entity from ORM table
	 * @access public
	 * @param int $id
	 * @return Object&
	 */
	public function loadEntity($id);

	/**
	 * Cancel editing entity
	 *
	 * @param int $id
	 * @access public
	 * @return boolean
	 */
	public function cancelEntity($id);

	/**
	 * Delete entity
	 *
	 * @param array $ids
	 * @access public
	 * @return boolean
	 */
	public function deleteEntity($ids);

	/**
	 * Storing entity by ORM table
	 *
	 * @access public
	 * @return mixed
	 */
	public function storeEntity();

	/**
	 * Publishing state changer for entities
	 *
	 * @access public
	 * @param int $idEntity
	 * @param string $state
	 * @return boolean
	 */
	public function publishEntities($idEntity, $state);
}

/**
 * Base concrete model for business logic
 * 
 * @package RESPONSIVIZER::FRAMEWORK::administrator::components::com_responsivizer
 * @subpackage framework
 * @subpackage model
 * @since 2.0
 */
class ResponsivizerModel extends JModel implements IResponsivizerModel {
	/**
	 * Application reference
	 * 
	 * @access protected
	 * @var Object
	 */
	protected $app;

	/**
	 * Main get data method
	 *
	 * @access public
	 * @return Object[]
	 */
	public function getData() {
		// Build query
		$query = $this->buildListQuery();
		$this->_db->setQuery($query, $this->getState('limitstart'), $this->getState('limit'));
		try {
			$result = $this->_db->loadObjectList();
			if ($this->_db->getErrorNum()) {
				throw new ResponsivizerException(JTEXT::sprintf('COM_RESPONSIVIZER_ERROR_RECORDS', $this->_db->getErrorMsg()), 'error');
			}
		} catch (ResponsivizerException $e) {
			$this->app->enqueueMessage($e->getMessage(), $e->getErrorLevel());
			$result = array();
		} catch (Exception $e) {
			$rvzrException = new ResponsivizerException($e->getMessage(), 'error');
			$this->app->enqueueMessage($rvzrException->getMessage(), $rvzrException->getErrorLevel());
			$result = array();
		}
		return $result;
	}

	/**
	 * Counter result set
	 *
	 * @access public
	 * @return int
	 */
	public function getTotal() {
		// Build query
		$query = $this->buildListQuery();
		$this->_db->setQuery($query);
		$result = count($this->_db->loadResultArray());

		return $result;
	}

	/**
	 * Load entity from ORM table
	 *
	 * @access public
	 * @param int $id
	 * @return Object&
	 */
	public function loadEntity($id) {
		// load table record
		$table = $this->getTable();

		try {
			if (!$table->load($id)) {
				throw new ResponsivizerException($this->_db->getErrorMsg(), 'error');
			}
		} catch (ResponsivizerException $e) {
			$this->setError($e);
			return false;
		} catch (Exception $e) {
			$ResponsivizerException = new ResponsivizerException($e->getMessage(), 'error');
			$this->setError($ResponsivizerException);
			return false;
		}
		return $table;
	}

	/**
	 * Cancel editing entity
	 *
	 * @param int $id
	 * @access public
	 * @return boolean
	 */
	public function cancelEntity($id) {
		// New record - do null e return true subito
		if (!$id) {
			return true;
		}

		$table = $this->getTable();
		try {
			if (!$table->load($id)) {
				throw new ResponsivizerException($table->getError(), 'error');
			}
			if (!$table->checkin()) {
				throw new ResponsivizerException($table->getError(), 'error');
			}
		} catch (ResponsivizerException $e) {
			$this->setError($e);
			return false;
		} catch (Exception $e) {
			$ResponsivizerException = new ResponsivizerException($e->getMessage(), 'error');
			$this->setError($ResponsivizerException);
			return false;
		}

		return true;
	}

	/**
	 * Delete entity
	 *
	 * @param array $ids
	 * @access public
	 * @return boolean
	 */
	public function deleteEntity($ids) {
		$table = $this->getTable();

		// Ciclo su ogni entity da cancellare
		foreach ($ids as $id) {
			try {
				if (!$table->delete($id)) {
					throw new ResponsivizerException($table->getError(), 'error');
				}
				// Only if table supports ordering
				if (property_exists($table, 'ordering')) {
					$table->reorder();
				}
			} catch (ResponsivizerException $e) {
				$this->setError($e);
				return false;
			} catch (Exception $e) {
				$ResponsivizerException = new ResponsivizerException($e->getMessage(), 'error');
				$this->setError($ResponsivizerException);
				return false;
			}
		}

		return true;
	}

	/**
	 * Storing entity by ORM table
	 *
	 * @access public
	 * @return mixed
	 */
	public function storeEntity() {
		$table = $this->getTable();
		try {
			if (!$table->bind($_POST, true)) {
				throw new ResponsivizerException($table->getError(), 'error');
			}

			if (!$table->check()) {
				throw new ResponsivizerException($table->getError(), 'error');
			}

			if (!$table->store(false)) {
				throw new ResponsivizerException($table->getError(), 'error');
			}
			// Only if table supports ordering
			if (property_exists($table, 'ordering')) {
				$table->reorder();
			}
		} catch (ResponsivizerException $e) {
			$this->setError($e);
			return false;
		} catch (Exception $e) {
			$ResponsivizerException = new ResponsivizerException($e->getMessage(), 'error');
			$this->setError($ResponsivizerException);
			return false;
		}
		return $table;
	}

	/**
	 * Publishing state changer for entities
	 *
	 * @access public
	 * @param int $idEntity
	 * @param string $state
	 * @return boolean
	 */
	public function publishEntities($idEntity, $state) {
		// Table load
		$table = $this->getTable($this->getName(), 'Table');
		if (isset($idEntity)) {
			try {
				if (!$table->load($idEntity)) {
					throw new ResponsivizerException($table->getError(), 'notice');
				}
				switch ($state) {
				case 'unpublish':
					$table->published = null;
					break;

				case 'publish':
					$table->published = 1;
					break;
				}

				if (!$table->store(true)) {
					throw new ResponsivizerException($table->getError(), 'notice');
				}
			} catch (ResponsivizerException $e) {
				$this->setError($e);
				return false;
			} catch (Exception $e) {
				$rvzrException = new ResponsivizerException($e->getMessage(), 'notice');
				$this->setError($rvzrException);
				return false;
			}
		}
		return true;
	}

	/**
	 * Class constructor
	 *
	 * @access public
	 * @param $config array
	 * @return Object&
	 */
	public function __construct($config = array()) {
		parent::__construct($config);
		$this->app = JFactory::getApplication();
	}
}
