<?php
// namespace administrator\components\com_responsivizer\views\cpanel;
/**
 * @package RESPONSIVIZER::CPANEL::administrator::components::com_responsivizer
 * @subpackage views
 * @subpackage cpanel
 * @author Joomla! Extensions Store
 * @copyright (C) 2013 - Joomla! Extensions Store
 * @license GNU/GPLv2 http://www.gnu.org/licenses/gpl-2.0.html  
 */
defined ( '_JEXEC' ) or die ( 'Restricted access' );

/**
 * CPanel view
 *
 * @package RESPONSIVIZER::CPANEL::administrator::components::com_responsivizer
 * @subpackage views
 * @subpackage cpanel
 * @since 2.0
 */
class ResponsivizerViewCpanel extends ResponsivizerView {

	/**
	 * Render iconset for cpanel
	 *
	 * @param $link string
	 * @param $image string
	 * @access private
	 * @return string
	 */
	private function getIcon($link, $image, $text, $target = '', $title = null, $class = null) {
		$mainframe = JFactory::getApplication ();
		$lang = JFactory::getLanguage ();
		?>
		<div style="float:<?php echo ($lang->isRTL()) ? 'right' : 'left'; ?>;">
			<div class="icon">
				<a <?php echo $title . $class;?> <?php echo $target;?> href="<?php echo JFilterOutput::ampReplace($link); ?>">
					<?php echo JHTML::_('image.site',  $image, '/components/com_responsivizer/images/', NULL, NULL, $text ); ?>
					<span><?php echo $text; ?></span>
				</a>
			</div>
		</div>
		<?php
	}
	
	/**
	 * Add the page title and toolbar.
	 *
	 * @since	1.6
	 */
	protected function addDisplayToolbar() {
		$doc = JFactory::getDocument();
		$doc->addStyleDeclaration('.icon-48-responsivizer{background-image:url("components/com_responsivizer/images/responsivizer-48x48.png")}');
		$doc->addStyleDeclaration('.icon-32-cpanel{background-image:url("components/com_responsivizer/images/icon-32-cpanel.png")}');
		JToolBarHelper::title( JText::_( 'COM_RESPONSIVIZER_CPANEL_TOOLBAR' ), 'responsivizer' );
		JToolBarHelper::custom('cpanel.display', 'cpanel', 'cpanel', 'COM_RESPONSIVIZER_CPANEL', false);
	}
	
	/**
	 * Control panel display
	 *        	
	 * @access public
	 * @param string $tpl
	 * @return void
	 */
	public function display($tpl = null) {
		$doc = JFactory::getDocument ();
		$this->loadJQuery($doc);
		$this->loadBootstrap($doc);
		$doc->addStylesheet ( JURI::root ( true ) . '/administrator/components/com_responsivizer/assets/css/cpanel.css' );
		$doc->addStylesheet ( JURI::root ( true ) . '/administrator/components/com_responsivizer/assets/css/emulator.css' );
		$doc->addScript ( JURI::root ( true ) . '/administrator/components/com_responsivizer/js/chart.js' );
		$doc->addScript ( JURI::root ( true ) . '/administrator/components/com_responsivizer/js/emulator.js' );
		$doc->addScript ( JURI::root ( true ) . '/administrator/components/com_responsivizer/js/cpanel.js' );
		
		// Inject js translations
		$translations = array (	'COM_RESPONSIVIZER_TOTAL_MODULES_CHART',
								'COM_RESPONSIVIZER_PUBLISHED_MODULES_CHART',
								'COM_RESPONSIVIZER_UNPUBLISHED_MODULES_CHART',
								'COM_RESPONSIVIZER_TOTAL_MOBILE_MODULES_CHART',
								'COM_RESPONSIVIZER_PUBLISHED_MOBILE_MODULES_CHART',
								'COM_RESPONSIVIZER_UNPUBLISHED_MOBILE_MODULES_CHART',
								'COM_RESPONSIVIZER_TOTAL_CACHED_PAGES_CHART',
								'COM_RESPONSIVIZER_PUBLISHED_CACHED_PAGES_CHART',
								'COM_RESPONSIVIZER_SPACER_CHARACTER',
								'COM_RESPONSIVIZER_RESOLUTION_CHARACTER',
								'COM_RESPONSIVIZER_CHOOSE_PHONE',
								'COM_RESPONSIVIZER_CHOOSE_TABLET',
								'COM_RESPONSIVIZER_SCREEN_SIZE',
								'COM_RESPONSIVIZER_PIXEL_RATIO'
		);
		$this->injectJsTranslations($translations, $doc);
		
		$livesite =  substr_replace(JURI::root(), "", -1, 1);
		$infoData = $this->getModel()->getData($this->get('registry'));
		$doc->addScriptDeclaration('var responsivizerChartData = ' . json_encode($infoData));
		
		$lists = $this->get('Lists');
		// Buffer delle icons
		ob_start ();
		$this->getIcon ( 'index.php?option=com_responsivizer&amp;task=config.display&amp;fieldset=template', 'icon-48-settings_template.png', JText::_ ( 'COM_RESPONSIVIZER_TEMPLATE_SETTINGS' ));
		$this->getIcon ( 'index.php?option=com_responsivizer&amp;task=config.display&amp;fieldset=plugin', 'icon-48-settings_plugins.png', JText::_ ( 'COM_RESPONSIVIZER_PLUGINS_SETTINGS' ));
		$this->getIcon ( 'index.php?option=com_responsivizer&amp;task=config.display&amp;fieldset=module', 'icon-48-settings_slideshow.png', JText::_ ( 'COM_RESPONSIVIZER_SLIDESHOW_SETTINGS' ));
		$this->getIcon ( 'index.php?option=com_responsivizer&task=modulesmanager.display', 'icon-48-modules_manager.png', JText::_ ( 'COM_RESPONSIVIZER_MODULES_MANAGER' ));
		$this->getIcon ( 'index.php?option=com_responsivizer&task=editortemplate.display', 'icon-48-template_editor.png', JText::_ ( 'COM_RESPONSIVIZER_EDITOR_TEMPLATE' ));
		$this->getIcon ( 'index.php?option=com_responsivizer&task=offlinecache.display', 'icon-48-offline.png', JText::_ ( 'COM_RESPONSIVIZER_OFFLINE_CACHE' ));
		$this->getIcon ( '#', 'icon-48-emulator.png', JText::_ ( 'COM_RESPONSIVIZER_EMULATOR' ), '', 'data-content="' . JText::_('COM_RESPONSIVIZER_START_EMULATOR') . '"', 'class="triggerEmulator"');
		$this->getIcon ( 'index.php?option=com_responsivizer&task=help.display', 'icon-48-help.png', JText::_ ( 'COM_RESPONSIVIZER_HELP' ));

		$contents = ob_get_clean ();
		 
		// Assign reference variables
		$this->icons = $contents;
		$this->livesite = $livesite;
		$this->componentParams = JComponentHelper::getParams('com_responsivizer');
		$this->infoData = $infoData;
		$this->updatesData = $this->getModel()->getUpdates($this->get('httpclient'));
		$this->lists = $lists;
		$this->currentVersion = JText::_('COM_RESPONSIVIZER_VERSION_COMPONENT');
		
		$doc->addScriptDeclaration('var responsiveTemplate = "' . $this->componentParams->get('template', 'responsivizer') . '"');
		
		// Aggiunta toolbar
		$this->addDisplayToolbar();
		
		// Output del template
		parent::display ();
	}
}